# -*- coding: utf-8 -*-

"""Top-level package for PyGantt."""

__author__ = """PyGantt Developers"""
__email__ = 'seth.mridul@gmail.com'
__version__ = '0.1.0'


from pygantt.pygantt import *
from pygantt.util import *
